# Custom Component Service Sample - developer-service

## SDK version: 2.0 (SDK as an npm package)

This custom component service serves the custom components needed by the developer Bots including FinancialBot, Parenting Bot, Retail Bot, Stocks Bot and Telco Bot.

## Custom Component Package format

A Component Package consists of the following:

- package.json
- main.js
- .npmignore
- component file(s)

### package.json

The package.json file needs to have a "main" entry that is entry point for the consuming wrapper application.  For example, the package.json "main" try is "main.js".

The package.json file has a devDependency to the "@oracle/bots-node-sdk".  The Bots Node SDK is used to test the component code.  The runtime version of the SDK is specified by the service wrapper project package.json.  In some runtime environment, if the service wrapper does not specify the sdk version, the verson specified by the component package devDependency will be used.

### main.js

main.js exports the settings of the package.  It has the following properties:

"components" - this is required property to specify the list of components the package exports.  The "components" property can be in one of the 2 formats below:

First, "components" can be an object with property names being component names, and property value being the corresponding component module.  For example:

```javascript
module.exports = {
  components: {
      'hello.world': require('./components/hello.world')
  }
};
```

This format is the same format as registry.js for users of prior samples.  You could simply rename registry.js to main.js.

Second, "components" can be an array of paths to component file or directory.  For example:

```javascript
module.exports = {
  components: [
    './components/hello.world'
  ]
};
```

or

```javascript
module.exports = {
  components: [
    './components'
  ]
};
```

The main.js file will thus export all component modules or file/directory as the component package content.

### .npmignore

The .npmignore file is used when exporting the Component Package (see below).  It simply excludes the .tgz file from being included within the package export.

### component file(s)

There could be 1 or more component file(s) in a package.  The component files could be in different sub-directories.  The component code exports a metadata() method and an invoke() method.  See the included example.  

## Export the Component Package

A Component Package can be exported by running `npm pack` into a .tgz file.  This .tgz file could then be used to construct full custom component service using different service wrappers.

## Upgrade Steps (Bots Node SDK from version 1.1+ to v2.0)

If your custom component service uses the SDK that is not packaged as npm module, you could upgrade using instructions below.  Your custom component service will continue to work without upgrade.  CHanging it to use the npm module version makes future upgrade easier.

### (Bots Custom Components) Replace the v1.0 SDK and supporting files with the v1.1 SDK

1. Remove the following files from your custom component service:
    - sdk.js
    - shell.js
    - MessageModel.js

1. In your component.js (or equivalent custom component service entry point):
    - Add the new SDK

    ```javascript
    const OracleBot = require('@oracle/bots-node-sdk');
    ```

    - change the express router as follows:

    ```javascript
    router.use(OracleBot.Middleware.customComponent({
        register: require('./registry').components
    }));
    ```

    See the [Bots Node SDK docs](/ext/nodesdk/) for more options of using the new SDK

1. In package.json add the SDK as dependency

    "@oracle/bots-node-sdk": "^2.0.0"
