"use strict"

var request = require('request');
var UserMap = {};
var AgentMap = {};
var integrationBackChannel = null;
var userChannel = null;

// Utilizes out of band messages for sending/receiving messages between live bots users
//
module.exports = {

    metadata: () => ({
        "name": "agent.switchboard",
        "properties": {
            "state": { "type": "string", "required": false }
        },
        "supportedActions": [
            "leave"
        ]
    }),

    invoke: (conversation, done) => {
        const userId = conversation.request().message.channelConversation.userId;

        const channelId = conversation.request().message.channelConversation.channelId;

        var messagePayload = conversation.messagePayload();

        // messages incoming from agent to AgentBot may be on the 1.0 channel.
        if (!messagePayload)
        {
            messagePayload = conversation.rawPayload();
        }
        var messageType = messagePayload.type;
        var text = messagePayload.text;

        conversation.logger().info("switchboard: AgentMap at invoke looks like: " + JSON.stringify(AgentMap));

        conversation.logger().info('switchboard: Text: ' + text + ', type: ' + messageType);
        var responsePayload = null;

        const isIntegrationBackChannelMessage= ((messageType && messageType.toLowerCase().includes("agent")) || channelId === integrationBackChannel);

        var reply = {};

        var agent = null;

        if (isIntegrationBackChannelMessage) // originating from the user/customer
        {
            //
            var agentResolved = false;

            conversation.logger().info('switchboard: isIntegrationBackChannelMessage');

            integrationBackChannel = channelId;
            var userState = UserMap[userId];
            if (messageType === 'agentRequest')
            {
                // look to see if an agent is available.  If so set them up together.
                //
                var foundAvailableAgent = false;
                for (var agentId in AgentMap)
                {
                    if (AgentMap.hasOwnProperty(agentId))
                    {
                        agent = AgentMap[agentId];
                        if (!agent.partner)
                        {
                            foundAvailableAgent = true;

                            //userState.name = conversation.variable("profile.firstName");
                            //userState.partner = agent.channelConversation;
                            //UserMap[userId] = userState;
                            agent.partnerName = messagePayload.userProfile.firstName; //conversation.variable("profile.firstName");
                            agent.partner = conversation.request().message.channelConversation;
                            agent.partnerChannelName = messagePayload.channelName;

                            responsePayload = {};
                            reply.messagePayload = responsePayload;
                            responsePayload.type = "agentRequestResponse";
                            responsePayload.text = agent.name + " is joining the conversation.";
                            responsePayload.status = "accepted";
                            responsePayload.channelName = agent.partnerChannelName;
                            reply.channelConversation = agent.partner;
                            conversation.reply(reply.messagePayload, reply.channelConversation);

                            reply = {};
                            responsePayload = {};
                            reply.messagePayload = responsePayload;
                            responsePayload.text = agent.partnerName + " would like to talk to you about: " + text;
                            responsePayload.type = "text";
                            reply.channelConversation = agent.channel;
                            conversation.reply(reply.messagePayload, reply.channelConversation);
                            break;
                        }
                    }
                }
                if (!foundAvailableAgent)
                {
                    responsePayload = {};
                    reply.messagePayload = responsePayload;
                    responsePayload.type = "agentRequestResponse";
                    responsePayload.text = "All agents are currently busy.  Please try again later.";
                    responsePayload.status = "rejected";
                    responsePayload.channelName = agent.partnerChannelName;
                    reply.channelConversation = conversation.request().message.channelConversation;
                    conversation.reply(reply.messagePayload, reply.channelConversation);
                }
                agentResolved = true;
            } else if (messageType === 'agentConversationHistory')
            {
                for (var eachAgent in AgentMap)
                {
                    if (AgentMap.hasOwnProperty(eachAgent))
                    {
                        agent = AgentMap[eachAgent];
                        if (agent.partner && (agent.partner.userId === userId))
                        {
                            agentResolved = true;
                            var historyText = text;
                            var history = messagePayload.conversationHistory;
                            history.forEach(function(obj) {
                                historyText += "\n";
                                if (obj.source && obj.source.toLowerCase() === 'bot')
                                {
                                    historyText += "\t\t";
                                }
                                historyText += (typeof obj.payload === 'string') ? obj.payload : JSON.stringify(obj.payload);
                            });

                            responsePayload = {};
                            reply.messagePayload = responsePayload;
                            responsePayload.type = "text";
                            responsePayload.text = historyText;
                            reply.channelConversation = agent.channel;
                            conversation.reply(reply.messagePayload, reply.channelConversation);
                            break;
                        }
                    }
                }
            } else if (messageType === 'agentRequestCancelled')
            {

            } else if (messageType === 'botTextMessagePayload' || messageType === 'text' || !messageType)
            {
                for (var eachAgent in AgentMap)
                {
                    if (AgentMap.hasOwnProperty(eachAgent))
                    {
                        agent = AgentMap[eachAgent];
                        if (agent.partner && (agent.partner.userId === userId))
                        {
                            agentResolved = true;
                            responsePayload = {};
                            reply.messagePayload = responsePayload;
                            responsePayload.type = "text";
                            responsePayload.text = agent.partnerName + " says: " + text;
                            reply.channelConversation = agent.channel;
                            conversation.reply(reply.messagePayload, reply.channelConversation);
                            break;
                        }
                    }
                }
            } else if (messageType === 'botConversationEnded')
            {
                for (var eachAgent in AgentMap)
                {
                    if (AgentMap.hasOwnProperty(eachAgent))
                    {
                        agent = AgentMap[eachAgent];
                        if (agent.partner && (agent.partner.userId === userId))
                        {
                            agentResolved = true;
                            responsePayload = {};
                            reply.messagePayload = responsePayload;
                            responsePayload.type = "text";
                            responsePayload.text = agent.partnerName + " has left the conversation.";
                            reply.channelConversation = agent.channel;
                            agent.partnerName = null;
                            agent.partner = null;
                            conversation.reply(reply.messagePayload, reply.channelConversation);
                            break;
                        }
                    }
                }
            }
            if (!agentResolved)
            {
                conversation.logger().info("switchboard: Associated agent for this request can no longer be found!");
                // this could be the dreaded infinite oops between two bots, where each bot will
                // always insist on replying to the other bot's oops message.
                // Eventually someone needs to just shut up.
                if (messageType === 'text' || !messageType)
                {
                    conversation.releaseTurn();
                } else
                {
                    conversation.reply("Oops the agent appears to have left the conversation or a message of type " +
                    messageType + " and text " + text + " was sent prematurely before agent contact could be established.");
                }
            }
        } else // originating from the agent in AgentBot
        {
            if (messageType === 'text' || !messageType)
            {
                agent = AgentMap[userId];
                if (!agent)
                {
                    agent = {};
                    // at this point still haven't asked for a preferred name in the Bot...
                    agent.name = "An agent"; // conversation.variable("profile.firstName");
                    agent.channel = conversation.request().message.channelConversation;
                    AgentMap[userId] = agent;
                }
                if (agent.partner)
                {
                    // TODO: postback buttons are so much better!
                    if (text && (text.toLowerCase().startsWith('leave') || text.toLowerCase().startsWith('bye')))
                    {
                        responsePayload = {};
                        reply.messagePayload = responsePayload;
                        responsePayload.type = "agentLeft";
                        responsePayload.text = conversation.variable("firstName") + " has left the conversation.";
                        responsePayload.channelName = agent.partnerChannelName;
                        reply.channelConversation = agent.partner;
                        agent.partnerName = null;
                        agent.partner = null;
                        conversation.reply(reply.messagePayload, reply.channelConversation);
                    } else
                    {
                        responsePayload = {};
                        reply.messagePayload = responsePayload;
                        responsePayload.type = "agent";
                        responsePayload.text = conversation.variable("firstName") + " says: " + text;
                        responsePayload.channelName = agent.partnerChannelName;
                        reply.channelConversation = agent.partner;
                        conversation.reply(reply.messagePayload, reply.channelConversation);
                    }
                } else
                {
                    //
                    conversation.logger().info("Non-integration back channel text message, just move to next state.");
                    if (!text || text.startsWith("Oops "))
                    {
                        conversation.keepTurn(false);
                    }
                    conversation.transition("leave");
                }
                if (text && text.toLowerCase().startsWith('bye'))
                {
                    conversation.reply("Goodbye " + conversation.variable("firstName"));
                    conversation.transition();
                    delete AgentMap[userId];
                    conversation.logger().info("AgentMap after agent says bye is : " + JSON.stringify(AgentMap));
                }
            } else
            {
                conversation.logger().info("All messages coming from the agent him/herself should be text for now");
                conversation.transition("leave");
            }
        }
        done();
    }
};
